/*
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 */

/*
 * Copyright 2010 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

/*
 * Copyright 2012 David Hoeppner.  All rights reserved.
 */

/*
 * This file contains functions related to getting and setting options
 * thought the getsockopt and setsockopt socket functions.
 */

#include <sys/types.h>
#include <sys/stream.h>
#define	_SUN_TPI_VERSION 2
#include <sys/tihdr.h>
#include <sys/xti_xtiopt.h>
#include <sys/xti_inet.h>
#include <sys/policy.h>

#include <inet/common.h>
#include <inet/dccp_impl.h>
#include <inet/ip.h>
#include <inet/optcom.h>
#include <netinet/ip.h>

#include <sys/cmn_err.h>

static int dccp_opt_default(queue_t *, int, int, uchar_t *);

/*
 * Supported options.
 */
opdes_t dccp_opt_arr[] = {
{ SO_DEBUG, SOL_SOCKET, OA_RW, OA_RW, OP_NP, 0, sizeof (int), 0 },
}; 

/*
 * Supported levels.
 */
optlevel_t dccp_valid_levels_arr[] = {
	SOL_SOCKET,
};

#define	DCCP_OPT_ARR_CNT	A_CNT(dccp_opt_arr)
#define	DCCP_VALID_LEVELS_CNT	A_CNT(dccp_valid_levels_arr)

uint_t dccp_max_optsize;

/*
 * Options database object.
 */
optdb_obj_t dccp_opt_obj = {
	dccp_opt_default,
	dccp_tpi_opt_get,
	dccp_tpi_opt_set,
	DCCP_OPT_ARR_CNT,
	dccp_opt_arr,
	DCCP_VALID_LEVELS_CNT,
	dccp_valid_levels_arr,
};

/*
 * Default value for certain options.
 */
int
dccp_opt_default(queue_t *q, int level, int name, uchar_t *ptr)
{
	dccp_stack_t	*dccps = Q_TO_DCCP(q)->dccp_dccps;
	int32_t		*il = (int32_t *)ptr;

	return (sizeof (int));
}

int
dccp_opt_get(conn_t *connp, int level, int name, uchar_t *ptr)
{
	dccp_t		*dccp = connp->conn_dccp;
	conn_opt_arg_t	coas;
	int		retval;

	coas.coa_connp = connp;
	coas.coa_ixa = connp->conn_ixa;
	coas.coa_ipp = &connp->conn_xmit_ipp;
	coas.coa_ancillary = B_FALSE;
	coas.coa_changed = 0;

	switch (level) {
	case SOL_SOCKET:
		break;
	case IPPROTO_TCP:
		break;
	case IPPROTO_IP:
		break;
	case IPPROTO_IPV6:
		break;
	}

	mutex_enter(&connp->conn_lock);
	retval = conn_opt_get(&coas, level, name, ptr);
	mutex_exit(&connp->conn_lock);

	return (retval);
}

/* ARGSUSED */
int
dccp_opt_set(conn_t *connp, uint_t optset_context, int level, int name,
    uint_t inlen, uchar_t *invalp, uint_t *outlenp, uchar_t *outvalp,
    void *thisdg_attrs, cred_t *cr)
{
	dccp_t		*dccp = connp->conn_dccp;
	dccp_stack_t	*dccps = dccp->dccp_dccps;
	conn_opt_arg_t	coas;
	int		*i1 = (int *)invalp;
	int		error;

	coas.coa_connp = connp;
	coas.coa_ancillary = B_FALSE;
	coas.coa_changed = 0;

	error = conn_opt_set(&coas, level, name, inlen, invalp,
	    B_FALSE, cr);
	if (error !=0) {
		*outlenp = 0;
		return (error);
	}

	return (0);
}
